"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.Collection = void 0;
const promises_1 = require("node:dns/promises");
const mongodb_1 = require("./mongodb");
const errors_1 = require("./errors");
const utils_1 = require("./utils");
const common_1 = require("@nestjs/common");
class Collection extends mongodb_1.MongoCollection {
    logger = new common_1.Logger(this.constructor.name);
    constructor(db, name, options) {
        super(db, name, options);
    }
    countDocuments(filter, options) {
        return super.countDocuments(filter, options);
    }
    upsert(filter, update, options) {
        return super.updateOne(filter, update, {
            upsert: true,
            ...options
        });
    }
    withDocumentLock(options) {
        const { filter, withLockedDocument } = options;
        return this.lock(options)
            .then(document => Promise.resolve(withLockedDocument(document)))
            .then(result => this.unlockWithResult(filter, result))
            .catch(async (error) => {
            if (!(error instanceof errors_1.DocumentPinnedError)) {
                await this.unlock(filter);
            }
            throw error;
        });
    }
    async lock(options) {
        const { filter, document } = options;
        const lock = {
            _lockPinned: true,
            _lockOwnerPid: process.pid,
            _lockOwnerHostname: process.env.HOSTNAME || 'localhost'
        };
        const update = {
            $set: lock,
            $setOnInsert: document
        };
        const filters = {
            ...filter,
            _lockPinned: { $exists: false }
        };
        const isDocumentPinned = await this.findOne({
            ...filter,
            _lockPinned: true
        });
        const pid = isDocumentPinned?._lockOwnerPid;
        const hostname = isDocumentPinned?._lockOwnerHostname;
        const isOwnerAlive = isDocumentPinned && hostname
            ? await (0, promises_1.resolve)(hostname)
                .then(() => true)
                .catch(() => false)
            : Promise.resolve(false);
        if (isDocumentPinned && (!isOwnerAlive || !(0, utils_1.isProcessRunning)(pid))) {
            this.logger.debug('Unlocking process due to a document pin:');
            this.logger.debug(`- is owner not alive? ${!isOwnerAlive}`);
            this.logger.debug(`- is process not running? ${!(0, utils_1.isProcessRunning)(pid)}`);
            await this.unlock(filter);
        }
        return super.findOneAndUpdate(filters, update, { upsert: true, returnDocument: 'after' })
            .then(result => result)
            .catch(error => {
            if (error.code === 11000) {
                throw new errors_1.DocumentPinnedError('Document failed to insert a pin due to index constraint');
            }
            throw error;
        });
    }
    unlock(filter) {
        const lock = {
            _lockPinned: true,
            _lockOwnerPid: true,
            _lockOwnerHostname: true
        };
        return super.updateOne(filter, {
            $unset: lock
        });
    }
    unlockWithResult(filters, result) {
        return this.unlock(filters).then(() => result);
    }
}
exports.Collection = Collection;
//# sourceMappingURL=collection.js.map